@@ -39,6 +39,13 @@ BOLOS_TAG_DERIVEPATH = 0x04
 BOLOS_TAG_DATASIZE = 0x05
 BOLOS_TAG_DEPENDENCY = 0x06
 
+def string_to_bytes(x):
+	import sys
+	if sys.version_info.major == 3:
+		return bytes(x, 'ascii')
+	else:
+		return bytes(x)
+
 def encodelv(v):
 	l = len(v)
 	s = b""
@@ -382,11 +389,18 @@ class HexLoader:
 		return result
 
 	def listApp(self, restart=True):
-		if restart:
-			data = b'\x0E'
+		if self.secure:
+			if restart:
+				data = b'\x0E'
+			else:
+				data = b'\x0F'
+			response = self.exchange(self.cla, 0x00, 0x00, 0x00, data)
 		else:
-			data = b'\x0F'
-		response = self.exchange(self.cla, 0x00, 0x00, 0x00, data)
+			if restart:
+				response = self.exchange(self.cla, 0xDE, 0x00, 0x00, b'')
+			else:
+				response = self.exchange(self.cla, 0xDF, 0x00, 0x00, b'')
+
 		if sys.version_info.major == 2:
 			response = bytearray(response)
 		#print binascii.hexlify(response[0])
@@ -439,7 +453,7 @@ class HexLoader:
 		item['totalAppSlots'] = (response[offset] << 24) | (response[offset + 1] << 16) | (response[offset + 2] << 8) | response[offset + 3]
 		return item
 
-	def load(self, erase_u8, max_length_per_apdu, hexFile, reverse=False, doCRC=True):
+	def load(self, erase_u8, max_length_per_apdu, hexFile, reverse=False, doCRC=True, targetId=None, targetVersion=None):
 		if (max_length_per_apdu > self.max_mtu):
 			max_length_per_apdu = self.max_mtu
 		initialAddress = 0
@@ -447,6 +461,12 @@ class HexLoader:
 			initialAddress = hexFile.minAddr()
 		sha256 = hashlib.new('sha256')
 		# stat by hashing the create app params to ensure complete app signature
+		if (targetId != None and (targetId&0xF) > 3):
+			if (targetVersion == None):
+				print("Target version is not set, application hash will not match!")
+				targetVersion=""
+			#encore targetId U4LE, and version string bytes
+			sha256.update(struct.pack('>I', targetId) + string_to_bytes(targetVersion))
 		if self.createappParams:
 			sha256.update(self.createappParams)
 		areas = hexFile.getAreas()
