@@ -34,8 +34,8 @@ def get_argparser():
 	parser.add_argument("--targetVersion", help="Set the chip target version")
 	parser.add_argument("--fileName", help="The application hex file to be loaded onto the device")
 	parser.add_argument("--icon", help="The icon content to use (hex encoded)")
-	parser.add_argument("--curve", help="""A curve on which BIP 32 derivation is locked ("secp256k1", "prime256r1", or
-"ed25519"), can be repeated""", action='append')
+	parser.add_argument("--curve", help="""A curve on which BIP 32 derivation is locked ("secp256k1", "prime256r1",
+"ed25519" or "bls12381g1"), can be repeated""", action='append')
 	parser.add_argument("--path", help="""A BIP 32 path to which derivation is locked (format decimal a'/b'/c), can be
 repeated""", action='append')
 	parser.add_argument("--path_slip21", help="""A SLIP 21 path to which derivation is locked""", action='append')
@@ -145,6 +145,8 @@ if __name__ == '__main__':
 				curveMask |= 0x02
 			elif curve == 'ed25519':
 				curveMask |= 0x04
+			elif curve == 'bls12381g1':
+				curveMask |= 0x08
 			else:
 				raise Exception("Unknown curve " + curve)
 
